==> flink-1.13.3-src.tgz: flink-1.13.3/flink-connectors/flink-connector-hive/src/main/java/org/apache/flink/table/functions/hive/util/HiveFunctionUtil.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.flink.table.functions.hive.util;

import org.apache.flink.annotation.Internal;
import org.apache.flink.table.functions.hive.FlinkHiveUDFException;
import org.apache.flink.table.types.DataType;
import org.apache.flink.table.types.logical.ArrayType;
import org.apache.flink.table.types.logical.LogicalType;
import org.apache.flink.table.types.logical.LogicalTypeRoot;

/** Util for Hive functions. */
@Internal
public class HiveFunctionUtil {
    public static boolean isSingleBoxedArray(DataType[] argTypes) {
        for (DataType dataType : argTypes) {
            if (HiveFunctionUtil.isPrimitiveArray(dataType)) {
                throw new FlinkHiveUDFException(
                        "Flink doesn't support primitive array for Hive functions yet.");
            }
        }

        return argTypes.length == 1 && HiveFunctionUtil.isArrayType(argTypes[0]);
    }

    private static boolean isPrimitiveArray(DataType dataType) {
        if (isArrayType(dataType)) {
            ArrayType arrayType = (ArrayType) dataType.getLogicalType();

            LogicalType elementType = arrayType.getElementType();
            return !(elementType.isNullable() || !isPrimitive(elementType));
        } else {
            return false;
        }
    }

    // This is copied from PlannerTypeUtils in flink-table-runtime-blink that we shouldn't depend on
    // TODO: remove this and use the original code when it's moved to accessible, dependable module
    private static boolean isPrimitive(LogicalType type) {
        switch (type.getTypeRoot()) {
            case BOOLEAN:
            case TINYINT:
            case SMALLINT:
            case INTEGER:
            case BIGINT:
            case FLOAT:
            case DOUBLE:
                return true;
            default:
                return false;
        }
    }

    private static boolean isArrayType(DataType dataType) {
        return dataType.getLogicalType().getTypeRoot().equals(LogicalTypeRoot.ARRAY);
    }
}

==> flink-1.14.4-src.tgz: flink-1.14.4/flink-connectors/flink-connector-hive/src/main/java/org/apache/flink/table/functions/hive/util/HiveFunctionUtil.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.flink.table.functions.hive.util;

import org.apache.flink.annotation.Internal;
import org.apache.flink.table.functions.hive.FlinkHiveUDFException;
import org.apache.flink.table.types.DataType;
import org.apache.flink.table.types.logical.ArrayType;
import org.apache.flink.table.types.logical.LogicalType;
import org.apache.flink.table.types.logical.LogicalTypeRoot;

/** Util for Hive functions. */
@Internal
public class HiveFunctionUtil {
    public static boolean isSingleBoxedArray(DataType[] argTypes) {
        for (DataType dataType : argTypes) {
            if (HiveFunctionUtil.isPrimitiveArray(dataType)) {
                throw new FlinkHiveUDFException(
                        "Flink doesn't support primitive array for Hive functions yet.");
            }
        }

        return argTypes.length == 1 && HiveFunctionUtil.isArrayType(argTypes[0]);
    }

    private static boolean isPrimitiveArray(DataType dataType) {
        if (isArrayType(dataType)) {
            ArrayType arrayType = (ArrayType) dataType.getLogicalType();

            LogicalType elementType = arrayType.getElementType();
            return !(elementType.isNullable() || !isPrimitive(elementType));
        } else {
            return false;
        }
    }

    // This is copied from PlannerTypeUtils in flink-table-runtime that we shouldn't depend on
    // TODO: remove this and use the original code when it's moved to accessible, dependable module
    private static boolean isPrimitive(LogicalType type) {
        switch (type.getTypeRoot()) {
            case BOOLEAN:
            case TINYINT:
            case SMALLINT:
            case INTEGER:
            case BIGINT:
            case FLOAT:
            case DOUBLE:
                return true;
            default:
                return false;
        }
    }

    private static boolean isArrayType(DataType dataType) {
        return dataType.getLogicalType().getTypeRoot().equals(LogicalTypeRoot.ARRAY);
    }
}
